@@ -22,7 +22,7 @@ OTHER DEALINGS IN THE SOFTWARE.
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from . import settings
-from .isort import SortImports
+from . import settings  # noqa: F401
+from .isort import SortImports  # noqa: F401
 
-__version__ = "4.2.5"
+__version__ = "4.3.5"
