@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 '''  Tool for sorting imports alphabetically, and automatically separated into sections.
 
 Copyright (C) 2013  Timothy Edmund Crosley
@@ -26,14 +25,11 @@ import glob
 import os
 import re
 import sys
-from concurrent.futures import ProcessPoolExecutor
 
 import setuptools
 
 from isort import SortImports, __version__
-from isort.settings import DEFAULT_SECTIONS, default, from_path, should_skip
-
-from .pie_slice import itemsview
+from isort.settings import DEFAULT_SECTIONS, WrapModes, default, from_path, should_skip
 
 INTRO = r"""
 /#######################################################################\
@@ -62,9 +58,14 @@ shebang_re = re.compile(br'^#!.*\bpython[23w]?\b')
 
 
 def is_python_file(path):
-    if path.endswith('.py'):
+    _root, ext = os.path.splitext(path)
+    if ext in ('.py', '.pyi'):
         return True
 
+    # Skip editor backup files.
+    if path.endswith('~'):
+        return False
+
     try:
         with open(path, 'rb') as fp:
             line = fp.readline(100)
@@ -91,13 +92,18 @@ def sort_imports(file_name, **arguments):
 
 def iter_source_code(paths, config, skipped):
     """Iterate over all Python source files defined in paths."""
+    if 'not_skip' in config:
+        config['skip'] = list(set(config['skip']).difference(config['not_skip']))
+
     for path in paths:
         if os.path.isdir(path):
             if should_skip(path, config, os.getcwd()):
                 skipped.append(path)
                 continue
 
-            for dirpath, dirnames, filenames in os.walk(path, topdown=True):
+            for dirpath, dirnames, filenames in os.walk(
+                    path, topdown=True, followlinks=True
+            ):
                 for dirname in list(dirnames):
                     if should_skip(dirname, config, dirpath):
                         skipped.append(dirname)
@@ -123,14 +129,14 @@ class ISortCommand(setuptools.Command):
 
     def initialize_options(self):
         default_settings = default.copy()
-        for (key, value) in itemsview(default_settings):
+        for key, value in default_settings.items():
             setattr(self, key, value)
 
     def finalize_options(self):
         "Get options from config files."
         self.arguments = {}
         computed_settings = from_path(os.getcwd())
-        for (key, value) in itemsview(computed_settings):
+        for key, value in computed_settings.items():
             self.arguments[key] = value
 
     def distribution_files(self):
@@ -165,10 +171,10 @@ class ISortCommand(setuptools.Command):
                 except IOError as e:
                     print("WARNING: Unable to parse file {0} due to {1}".format(python_file, e))
         if wrong_sorted_files:
-            exit(1)
+            sys.exit(1)
 
 
-def create_parser():
+def parse_args(argv=None):
     parser = argparse.ArgumentParser(description='Sort Python import definitions alphabetically '
                                                  'within logical sections.')
     inline_args_group = parser.add_mutually_exclusive_group()
@@ -228,7 +234,7 @@ def create_parser():
                         help="Forces line endings to the specified value. If not set, values will be guessed per-file.")
     parser.add_argument('-ls', '--length-sort', help='Sort imports by their string length.',
                         dest='length_sort', action='store_true')
-    parser.add_argument('-m', '--multi-line', dest='multi_line_output', type=int, choices=[0, 1, 2, 3, 4, 5],
+    parser.add_argument('-m', '--multi-line', dest='multi_line_output', type=int, choices=range(len(WrapModes)),
                         help='Multi line output (0-grid, 1-vertical, 2-hanging, 3-vert-hanging, 4-vert-grid, '
                         '5-vert-grid-grouped, 6-vert-grid-grouped-no-comma).')
     inline_args_group.add_argument('-nis', '--no-inline-sort', dest='no_inline_sort', action='store_true',
@@ -245,7 +251,7 @@ def create_parser():
                         help='Force sortImports to recognize a module as being part of the current python project.')
     parser.add_argument('-q', '--quiet', action='store_true', dest="quiet",
                         help='Shows extra quiet output, only errors are outputted.')
-    parser.add_argument('-r', '--remove-import', dest='remove_imports', action='append',
+    parser.add_argument('-rm', '--remove-import', dest='remove_imports', action='append',
                         help='Removes the specified import from all files.')
     parser.add_argument('-rc', '--recursive', dest='recursive', action='store_true',
                         help='Recursively look for Python files of which to sort imports')
@@ -281,16 +287,21 @@ def create_parser():
                         help='Tells isort to ignore whitespace differences when --check-only is being used.')
     parser.add_argument('-y', '--apply', dest='apply', action='store_true',
                         help='Tells isort to apply changes recursively without asking')
+    parser.add_argument('--unsafe', dest='unsafe', action='store_true',
+                        help='Tells isort to look for files in standard library directories, etc. '
+                             'where it may not be safe to operate in')
     parser.add_argument('files', nargs='*', help='One or more Python source files that need their imports sorted.')
 
-    arguments = {key: value for key, value in itemsview(vars(parser.parse_args())) if value}
+    arguments = {key: value for key, value in vars(parser.parse_args(argv)).items() if value}
     if 'dont_order_by_type' in arguments:
         arguments['order_by_type'] = False
+    if arguments.pop('unsafe', False):
+        arguments['safety_excludes'] = False
     return arguments
 
 
-def main():
-    arguments = create_parser()
+def main(argv=None):
+    arguments = parse_args(argv)
     if arguments.get('show_version'):
         print(INTRO)
         return
@@ -325,31 +336,26 @@ def main():
         num_skipped = 0
         if config['verbose'] or config.get('show_logo', False):
             print(INTRO)
+
         jobs = arguments.get('jobs')
         if jobs:
-            executor = ProcessPoolExecutor(max_workers=jobs)
-
-            for sort_attempt in executor.map(functools.partial(sort_imports, **arguments), file_names):
-                if not sort_attempt:
-                    continue
-                incorrectly_sorted = sort_attempt.incorrectly_sorted
-                if arguments.get('check', False) and incorrectly_sorted:
-                    wrong_sorted_files = True
-                if sort_attempt.skipped:
-                    num_skipped += 1
+            import multiprocessing
+            executor = multiprocessing.Pool(jobs)
+            attempt_iterator = executor.imap(functools.partial(sort_imports, **arguments), file_names)
         else:
-            for file_name in file_names:
-                try:
-                    sort_attempt = SortImports(file_name, **arguments)
-                    incorrectly_sorted = sort_attempt.incorrectly_sorted
-                    if arguments.get('check', False) and incorrectly_sorted:
-                        wrong_sorted_files = True
-                    if sort_attempt.skipped:
-                        num_skipped += 1
-                except IOError as e:
-                    print("WARNING: Unable to parse file {0} due to {1}".format(file_name, e))
+            attempt_iterator = (sort_imports(file_name, **arguments) for file_name in file_names)
+
+        for sort_attempt in attempt_iterator:
+            if not sort_attempt:
+                continue
+            incorrectly_sorted = sort_attempt.incorrectly_sorted
+            if arguments.get('check', False) and incorrectly_sorted:
+                wrong_sorted_files = True
+            if sort_attempt.skipped:
+                num_skipped += 1
+
         if wrong_sorted_files:
-            exit(1)
+            sys.exit(1)
 
         num_skipped += len(skipped)
         if num_skipped and not arguments.get('quiet', False):
